%AMC1210_SIM.m
%This file simulates the AMC1210 functionality
%with a resolver.  It requires the use of several other
%functions included in this package.  MATLAB
%must have a path added to the location of this and
%the corresponding files.  See the Read_me.doc
%for further details.
N_samp = 1040e3;        % number of samples
index = 1:N_samp;pi = 3.14159;Vp = 2.0; 
f_carrier = 8e3;        % carrier frequency
f_clk = 32e6;           % clock frequency
CLK_DIV = 4;            % clock divider            
N_pat=f_clk/(f_carrier*CLK_DIV); % length of pattern register                
f_data=f_clk/CLK_DIV;   % frequency of digital output from PWM1 and 2
f_mod = 16e6;           % modulator frequency
f_motor = 100;          % motor frequency
N = 1;
SOSR = 125;
IOSR = (f_mod*N)/(f_carrier*SOSR);

y=sigpat(N_pat);      % generate pattern for carrier frequency
check_params( f_carrier,f_clk,CLK_DIV,N_pat,f_mod,N,SOSR,IOSR );

% Now simulate the resolver signal
res_out_sin= Vp*sin(2*pi*f_carrier*index/f_mod).*sin(2*pi*f_motor*index/f_mod);
res_out_cos= Vp*sin(2*pi*f_carrier*index/f_mod).*cos(2*pi*f_motor*index/f_mod);


sin_mod = modout(res_out_sin);      % "modout" simulates the modulator output signal
cos_mod = modout(res_out_cos);

out_sin = sincfilt(3,SOSR,sin_mod); % "sincfilt" similates the AMC1210 sinc. filter
out_cos = sincfilt(3,SOSR,cos_mod);

cal_shift_sin = calres(out_sin,f_mod/(f_carrier*SOSR)); % "calres" simulates the calibration
cal_shift_cos = calres(out_cos,f_mod/(f_carrier*SOSR)); % procedure that the AMC1210 performs

% "demodulate" rectifies and integrates
demod_sin = demodulate(out_sin, f_mod/(f_carrier*SOSR), cal_shift_sin, IOSR);   
demod_cos = demodulate(out_cos, f_mod/(f_carrier*SOSR), cal_shift_cos, IOSR);   

bitshift_sin = ceil(log2(max(demod_sin)-min(demod_sin)) - 16);
bitshift_cos = ceil(log2(max(demod_cos)-min(demod_cos)) - 16);

bitshift = max(bitshift_sin,bitshift_cos);   % bitshift tells how many bits
                                             % need to be shifted to get the
                                             % maximum amplitude
dem_sin_shift = abs(demod_sin);              % when only using 16 bit data
dem_cos_shift = abs(demod_cos);              % This value can be programmed 
                                             % into bits SH4-SH0 in the
                                             % integrator parameter
                                             % register
dem_sin_shift = dem_sin_shift./power(2,bitshift); % remove shifted bits
dem_cos_shift = dem_cos_shift./power(2,bitshift);

dem_sin_shift = dem_sin_shift.*sign(demod_sin); % put negative sign back
dem_cos_shift = dem_cos_shift.*sign(demod_cos);

% The following code is only for generating graphs
figure;
subplot(2,1,1);
plot(y);
xlim([0 N_pat]);
xlabel('Pattern Bits');
Title('Signal Generator Pattern');
subplot(2,1,2);
index2 = 1:N_pat;
plot([index2/(N_pat*f_carrier)],2.0*sin(2*pi*(index2)/N_pat));
Title('Filtered Carrier Cycle');
xlim([0 (index2(N_pat)/(N_pat*f_carrier))]);
xlabel('Time');
figure;
subplot(4,1,1);
plot([index/(f_mod)],res_out_sin);
hold;
plot([index/(f_mod)],res_out_cos,'r');
xlim([0 1/f_motor]);
Title('ADS1205 input signal');
ylabel('Volts');
subplot(4,1,2);
plot([(index(1:(N_samp/SOSR)))/(f_mod/SOSR)],out_sin);
hold;
plot([(index(1:(N_samp/SOSR)))/(f_mod/SOSR)],out_cos,'r');
xlim([0 1/f_motor]);
Title('Filtered modulator data');
ylabel('Code');
subplot(4,1,3);
plot([(index(1:(N_samp/(SOSR*IOSR))-1))/(f_mod/(SOSR*IOSR))],demod_sin);
hold;
plot([(index(1:(N_samp/(SOSR*IOSR))-1))/(f_mod/(SOSR*IOSR))],demod_cos,'r');
xlim([0 1/f_motor]);
Title('AMC1210 Output data');
ylabel('Code');
subplot(4,1,4);
plot([(index(1:(N_samp/(SOSR*IOSR))-1))/(f_mod/(SOSR*IOSR))],dem_sin_shift);
hold;
plot([(index(1:(N_samp/(SOSR*IOSR))-1))/(f_mod/(SOSR*IOSR))],dem_cos_shift,'r');
Title('AMC1210 Output data 16 bit mode');
xlim([0 1/f_motor]);
ylim([-32768 32768]);
xlabel('Time');
ylabel('Code');
legend('Sine - Ch. 1', 'Cosine - Ch. 2');
